/* Building Patrol Script v2.0 

Big Dawg KS 
 6/19/10 

Paramaters: 
 1. Object - patroling unit 
 2. Object - building to patrol 
 3. Array - buildingPos waypoints; 
    each waypoint may consist of either just a buildingPos (number), or an array of a buildingPos & direction to face 
 4. Boolean - true to setPos unit to first WP (used to start unit inside building) 
 5. Array/Number (Optional) - break to WP(buildingPos)/dir pair, or just a buildingPos, unit will move here when alerted 
 6. Number (Optional) - probablity unit has to crouch after being alerted, otherwise unit will stand 
 7. String (Optional) - condition for which to cease patrol; exits the script 

Description: 
 Give a unit a path to patrol inside of a building. Path consists of an array of waypoints. A waypoint consists of either a 
 buildingPos index, or an array containing a buildingPos index and a direction. An additional waypoint can be passed 
 that the unit will move to when alerted. Once the unit returns to SAFE, they will resume their patrol. Additional 
 parameters determine whether or not to teleport the unit to their first waypoint (useful to initialize unit on their patrol) 
 and the probability that the unit will crouch when alerted 

Examples: 
 [soldier1,structure,[[3,20],[4,90]],true] execVM "buildPatrol.sqf"; 
 [soldier1,structure,[5,7,1,4],false,4,0] execVM "buildPatrol.sqf"; 
 [soldier1,structure,[2,[3,15],5],false,[2,10],1] execVM "buildPatrol.sqf"; 
 [soldier1,structure,[[1,45],5,[2,5],3,[4,60]],true,[2,5],0.8,"player in list BLUFORDetected"] execVM "buildPatrol.sqf"; 

-------------------------------------------------------------------------------------------------------------------------------------------------------*/ 

// start initializing params 
_unit = _this select 0; 
if(local _unit)then{ 
_building = _this select 1; 
_wps = _this select 2; 
_start = _this select 3; 

// unit will start in the building 
if(_start)then{ 
    _wpp = _wps select 0; 
    if(typeName _wpp == "ARRAY")then{ 
        _unit setPos (_building buildingPos (_wpp select 0)); 
        _unit setDir (_wpp select 1); 
    }else{ 
        _unit setPos (_building buildingPos _wpp); 
    }; 
    _unit setBehaviour "SAFE"; 
}; 

// finish initializing varaibles & params 
_awp = "Null"; 
_crouch = 0.5; 
_cond = "false"; 
if(count _this >= 5)then{_awp = _this select 4}; 
if(count _this >= 6)then{_crouch = _this select 5}; 
if(count _this == 7)then{_cond = _this select 6}; 

// ensure that the condition evaluates to a boolean 
_testCond = { 
    private["_test","_testType"]; 
    _test = call compile _this; 
    _testType = typeName _test; 
    if(format ["%1",_testType] != "BOOL")then{_test = false}; 
    _test 
}; 

_unit setUnitPos "UP"; 
_cwp = 0; 
_pos = _building buildingPos 0; 
_dir = 0; 
while{alive _unit && !(_cond call _testCond)}do{ 

    if(unitPos _unit == "MIDDLE")then{_unit setUnitPos "UP"}; 

    // move to next WP 
    if(count _wps > 1 || !_start)then{ 
        _wpp = _wps select _cwp; 
        _wp = _wpp; 
        _dir = "null"; 
        if(typeName _wpp == "ARRAY")then{ 
            _wp = _wpp select 0; 
            _dir = _wpp select 1; 
        }; 
        _pos =  (_building buildingPos _wp); 
        _unit doMove _pos; 
        _unit doWatch objNull; 
    }; 

    // wait until unit has arrived at WP 
    waitUntil{ 
        ((_unit modelToWorld [0,0,0]) distance _pos <= 1.5)||(behaviour _unit != "SAFE" && format ["%1",_awp] != "Null")||(!alive _unit)||(_cond call _testCond) 
    }; 

    // tell unit to watch specified direction 
    if(format ["%1",_dir] != "null")then{ 
        _watchPos = [(_pos select 0)+(10*sin _dir),(_pos select 1)+(10*cos _dir),_pos select 2]; 
        _unit doWatch _watchPos; 
    }; 

    // break out of patrol if a break-out WP is defined 
    if(alive _unit && behaviour _unit != "SAFE" && format ["%1",_awp] != "Null" && !(_cond call _testCond))then{ 
        _wp = _awp; 
        _dir = "null"; 
        if(typeName _awp == "ARRAY")then{ 
            _wp = _awp select 0; 
            _dir = _awp select 1; 
        }; 
        _pos = (_building buildingPos _wp); 
        _unit doMove _pos; 
        _unit doWatch objNull; 
        waitUntil{((_unit modelToWorld [0,0,0]) distance _pos <= 1.5) || !(alive _unit)||(_cond call _testCond)}; 

        // if unit returns to SAFE, resume the patrol 
        while{alive _unit && behaviour _unit != "SAFE" && !(_cond call _testCond)}do{ 
            if((_crouch > random 1)&&(unitPos _unit != "MIDDLE"))then{_unit setUnitPos "MIDDLE"}; 
            if(format ["%1",_dir] != "null")then{ 
                _watchPos = [(_pos select 0)+(10*sin _dir),(_pos select 1)+(10*cos _dir),_pos select 2]; 
                _unit doWatch _watchPos; 
            }; 
            sleep (15 + random 25); 
            if(behaviour _unit == "SAFE")then{sleep (7 + random 5)}; 
        }; 
    }; 
    if((_crouch > random 1)&&(unitPos _unit != "MIDDLE")&&(behaviour _unit != "SAFE"))then{_unit setUnitPos "MIDDLE"; sleep (5 + random 3)}; 
    sleep (4 + random 13); 
    if(_cwp < ((count _wps)-1))then{_cwp = _cwp + 1}else{_cwp = 0}; 
}; // while alive 

// cease patrol 
if(alive _unit)then{ 
    _unit doFollow (leader group _unit); 
    _unit setUnitPos "AUTO"; 
    _unit doWatch objNull; 
}; 

}; // local unit  